import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from binstar_client.utils.notebook import Downloader
from binstar_client.utils.test.utils import data_dir


files = {'files': [
    {'basename': 'notebook', 'version': '1'},
    {'basename': 'notebook', 'version': '2'},
    {'basename': 'data', 'version': '2'}
]}


class DownloaderTestCase(unittest.TestCase):
    def test_ensure_location(self):
        aserver_api = mock.MagicMock()
        aserver_api.package = mock.MagicMock(return_value=files)

        downloader = Downloader(aserver_api, 'username', 'notebook')
        self.assertEqual(downloader.list_files()[0]['version'], '2')
        self.assertEqual(downloader.list_files()[1]['version'], '2')

    def test_can_download(self):
        package_1 = {'basename': 'notebook.ipynb'}
        package_2 = {'basename': 'NOEXIST'}
        downloader = Downloader('binstar', 'username', 'notebook')
        downloader.output = data_dir('')
        self.assertTrue(not downloader.can_download(package_1))
        self.assertTrue(downloader.can_download(package_1, True))
        self.assertTrue(downloader.can_download(package_2))

    def test_list_old_files(self):
        old_files = {'files': [{
            'basename': 'old-notebook',
            'version': '1.0.0',
            'upload_time': '2015-04-02 22:32:31.253000+00:00'
        }]}
        aserver_api = mock.MagicMock()
        aserver_api.package = mock.MagicMock(return_value=old_files)

        downloader = Downloader(aserver_api, 'username', 'notebook')
        self.assertEqual(downloader.list_files()[0]['version'], '1.0.0')

if __name__ == '__main__':
    unittest.main()



import inspect
import os
import tarfile
from tempfile import SpooledTemporaryFile
from binstar_client.errors import BinstarError


class CondaProject(object):
    # TODO: This class will be moved into Anaconda-Project
    def __init__(self, project_path, *args, **kwargs):
        self.project_path = project_path
        self._name = None
        self._tar = None
        self._size = None
        self.pfiles = []
        self.metadata = {
            'summary': kwargs.get('summary', None),
            'description': kwargs.get('description', None),
            'version': kwargs.get('version', None)
        }
        self.metadata = dict((k, v) for k, v in self.metadata.items() if v)

    def tar_it(self, fd=SpooledTemporaryFile()):
        with tarfile.open(mode='w', fileobj=fd) as tar:
            for pfile in self.pfiles:
                tar.add(pfile.fullpath, arcname=pfile.relativepath)
        fd.seek(0)
        self._tar = fd
        return fd

    def to_project_creation(self):
        return {
            'name': self.name,
            'access': 'public',
            'profile': {
                'description': self.metadata.get('description', ''),
                'summary': self.metadata.get('summary', ''),
            }
        }

    def to_stage(self):
        return {
            'basename': self.basename,
            'configuration': self.configuration,
        }

    @property
    def tar(self):
        if self._tar is None:
            self.tar_it()
        return self._tar

    @property
    def configuration(self):
        output = self.metadata.get('configuration', {})
        output.update({
            'size': self.size,
            'num_of_files': self.get_file_count()
        })
        return output

    def get_file_count(self):
        if os.path.isfile(self.project_path):
            return 1
        else:
            return len(self.pfiles)

    @property
    def basename(self):
        return "{}.tar".format(self.name)

    @property
    def size(self):
        if self._size is None:
            spos = self._tar.tell()
            self._tar.seek(0, os.SEEK_END)
            self._size = self._tar.tell() - spos
            self._tar.seek(spos)
        return self._size

    @property
    def name(self):
        if self._name is None:
            self._name = self._get_project_name()
        return self._name

    def _get_project_name(self):
        if os.path.isdir(self.project_path):
            return os.path.basename(os.path.abspath(self.project_path))
        else:
            return os.path.splitext(os.path.basename(self.project_path))[0]


class PFile(object):
    def __init__(self, **kwargs):
        self.fullpath = kwargs.get('fullpath', None)
        self.basename = kwargs.get('basename', None)
        self.relativepath = kwargs.get('relativepath', None)
        self.size = kwargs.get('size', None)
        self.populate()

    def __str__(self):
        if self.is_dir():
            return self.relativepath
        else:
            return "[{}] {}".format(self.size, self.relativepath)

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        return self.fullpath == other.fullpath

    def is_dir(self):
        return os.path.isdir(self.fullpath)

    def validate(self, validator):
        if inspect.isfunction(validator):
            return validator(basename=self.basename,
                             relativepath=self.relativepath,
                             fullpath=self.fullpath)
        elif inspect.isclass(validator):
            return validator(self)()
        raise BinstarError("Invalid validator {}".format(validator))

    def populate(self):
        if self.size is None:
            self.size = os.stat(self.fullpath).st_size
        if self.basename is None:
            self.basename = os.path.basename(self.fullpath)

    def to_dict(self):
        return {
            'basename': self.basename,
            'size': self.size,
            'relativepath': self.relativepath
        }



from __future__ import unicode_literals
# Standard library imports
import unittest

# Third party imports
from mock import patch

# Local imports
from binstar_client import errors
from binstar_client.scripts.cli import main
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch

try:
    import requests_kerberos
except ImportError:
    have_kerberos = False
else:
    have_kerberos = True


class Test(CLITestCase):
    @patch('binstar_client.commands.login.store_token')
    @patch('getpass.getpass')
    @patch('binstar_client.commands.login.input')
    @urlpatch
    def test_login(self, urls, input, getpass, store_token):
        input.return_value = 'test_user'
        getpass.return_value = 'password'

        urls.register(path='/', method='HEAD', status=200)
        urls.register(path='/authentication-type', content='{"authentication_type": "password"}')

        auth = urls.register(method='POST', path='/authentications', content='{"token": "a-token"}')
        main(['--show-traceback', 'login'], False)
        self.assertIn('login successful', self.stream.getvalue())

        auth.assertCalled()

        self.assertIn('Authorization', auth.req.headers)
        self.assertIn('Basic ', auth.req.headers['Authorization'])

        self.assertTrue(store_token.called)
        self.assertEqual(store_token.call_args[0][0], 'a-token')

    @patch('binstar_client.commands.login.store_token')
    @patch('getpass.getpass')
    @patch('binstar_client.commands.login.input')
    @urlpatch
    def test_login_compatible(self, urls, input, getpass, store_token):
        input.return_value = 'test_user'
        getpass.return_value = 'password'

        urls.register(path='/', method='HEAD', status=200)
        urls.register(path='/authentication-type', status=404)

        auth = urls.register(method='POST', path='/authentications', content='{"token": "a-token"}')
        main(['--show-traceback', 'login'], False)
        self.assertIn('login successful', self.stream.getvalue())

        auth.assertCalled()

        self.assertIn('Authorization', auth.req.headers)
        self.assertIn('Basic ', auth.req.headers['Authorization'])

        self.assertTrue(store_token.called)
        self.assertEqual(store_token.call_args[0][0], 'a-token')

    @unittest.skipIf(have_kerberos, "prompts user to install requests-kerberos")
    @urlpatch
    def test_login_kerberos_missing(self, urls):
        urls.register(path='/', method='HEAD', status=200)
        urls.register(path='/authentication-type', content='{"authentication_type": "kerberos"}')

        with self.assertRaisesRegexp(errors.BinstarError, 'conda install requests-kerberos'):
            main(['--show-traceback', 'login'], False)

    @unittest.skipIf(not have_kerberos, "requires requests-kerberos")
    @patch('binstar_client.commands.login.store_token')
    @urlpatch
    def test_login_kerberos(self, urls, store_token):

        urls.register(path='/', method='HEAD', status=200)
        urls.register(path='/authentication-type', content='{"authentication_type": "kerberos"}')

        def update_authentications():
            # The second request should succeed
            auth_success = urls.register(
                method='POST',
                path='/authentications',
                content='{"token": "a-token"}',
                headers={'WWW-Authenticate': 'Negotiate STOKEN'},
            )

        auth_fail = urls.register(method='POST', path='/authentications', status=401, headers={'WWW-Authenticate': 'Negotiate'}, side_effect=update_authentications)

        main(['--show-traceback', 'login'], False)
        self.assertIn('login successful', self.stream.getvalue())

        urls.assertAllCalled()
        self.assertTrue(store_token.called)
        self.assertEqual(store_token.call_args[0][0], 'a-token')
